#
# mouse_gui.py - GUI front end code for mouse configuration
#
# Brent Fox <bfox@redhat.com>
#
# Copyright 2002 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#!/usr/bin/python2.2

import string
import gtk
import gobject
import sys
import os
import rhpl.mouse as mouse
import kudzu

from rhpl.firstboot_gui_window import FirstbootGuiWindow

##
## I18N
## 
from rhpl.translate import _, N_
import rhpl.translate as translate
translate.textdomain ("redhat-config-mouse")

class MouseWindow(FirstbootGuiWindow):
    #You must specify a runPriority for the order in which you wish your module to run
    runPriority = 30
    moduleName = N_("Mouse")
    moduleClass = "reconfig"
    windowTitle = N_("Mouse Configuration")
    htmlTag = "mouse"
    commentTag = N_("Configure the mouse")
    shortMessage = N_("Select the appropriate mouse for the system.")

    def getNext(self):
        pass
    
    def destroy(self, args):
        gtk.mainquit()
    
    def setupScreen(self):
        self.mouse = mouse.Mouse(skipProbe = 1)
        self.mouse.read()
        mouseDict = self.mouse.mouseToMouse()
        probedMouse = self.mouse.info["FULLNAME"]

        self.Xemu3CheckButton = gtk.CheckButton(_("Emulate 3 button click"))

        self.mouseStore = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING,
                                        gobject.TYPE_STRING, gobject.TYPE_INT)
        self.deviceStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)

        #Add icon to the top frame
        p = None
        try:
            p = gtk.gdk.pixbuf_new_from_file("../pixmaps/gnome-mouse.png")
        except:
            try:
                p = gtk.gdk.pixbuf_new_from_file("/usr/share/redhat-config-mouse/pixmaps/gnome-mouse.png")
            except:
                pass

        if p:
            self.icon = gtk.Image()
            self.icon.set_from_pixbuf(p)

        currentDev = self.mouse.getDevice()
        currentMouse, emulate3 = self.mouse.get()

        self.mouseView = gtk.TreeView(self.mouseStore)
        self.col = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=0)
        self.mouseView.append_column(self.col)
        self.mouseView.set_property("headers-visible", gtk.FALSE)
        selection = self.mouseView.get_selection()
        selection.connect("changed", self.selectMouse)

        self.deviceView = gtk.TreeView(self.deviceStore)
        col = gtk.TreeViewColumn(_("_Device"), gtk.CellRendererText(), text=0)
        self.deviceView.append_column(col)

        self.mouseViewSW = gtk.ScrolledWindow()
        self.mouseViewSW.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.mouseViewSW.set_shadow_type(gtk.SHADOW_IN)
        self.mouseViewSW.add(self.mouseView)

        self.serialButton = gtk.Button(_("Serial devices..."))
        self.serialButton.connect("clicked", self.on_serialButton_clicked)
        self.serialButton.set_sensitive(gtk.FALSE)

        hbox = gtk.HBox()
        hbox.pack_start(self.Xemu3CheckButton, gtk.TRUE)
        hbox.pack_start(self.serialButton, gtk.FALSE)

        self.myVbox = gtk.VBox()
        self.myVbox.set_spacing(5)
        self.myVbox.pack_start(self.mouseViewSW, gtk.TRUE)
        self.myVbox.pack_start(hbox, gtk.FALSE, 3)

        self.setupMice()
        self.setupDeviceList()
        self.setCurrent(currentDev, currentMouse, emulate3)


        self.serialDlg = gtk.Dialog(_("Serial Devices"))
        self.serialDlg.set_border_width(10)
        label = gtk.Label(_("Select the correct serial device for the mouse."))
        label.set_alignment(0.0, 0.5)
        self.serialDlg.vbox.set_spacing(5)
        self.serialDlg.vbox.pack_start(label, gtk.FALSE)
        self.serialDlg.vbox.pack_start(self.deviceView, gtk.TRUE)
        self.serialDlg.add_button(gtk.STOCK_OK, 0)

    def setupMice(self):
        # go though and find all the makes that have more than 1 mouse
        toplevels = {}

        for key, value in self.mouse.available().items():
            make = string.split(_(key), ' - ')[0]
            if toplevels.has_key(make):
                toplevels[make] = toplevels[make] + 1
            else:
                toplevels[make] = 1

        # for each toplevel that has more than one mouse, make a parent
        # node for it.
        for make, count in toplevels.items():
            if count > 1:
                parent = self.mouseStore.append(None)
                self.mouseStore.set_value(parent, 0, make)
                toplevels[make] = parent
            else:
                del toplevels[make]
                
        # now go and add each child node
        for key, value in self.mouse.available().items():
            fields = string.split(_(key), ' - ')
            make = fields[0]
            model = fields[1]
            parent = toplevels.get(make)
            iter = self.mouseStore.append(parent)
            # if there is a parent, put only the model in the tree
            if parent:
                self.mouseStore.set_value(iter, 0, model)
            else:
                # otherwise, put the full device there.
                self.mouseStore.set_value(iter, 0, "%s %s" % tuple(fields))
            self.mouseStore.set_value(iter, 1, key)
            self.mouseStore.set_value(iter, 2, value[2])
            self.mouseStore.set_value(iter, 3, value[3])

    def on_serialButton_clicked(self, *args):
        self.serialDlg.show_all()
        rc = self.serialDlg.run()
        self.serialDlg.hide()

    def selectMouse(self, selection, *args):
        model, iter = selection.get_selected()
        type = self.mouseStore.get_value(iter, 2)
        emu3 = self.mouseStore.get_value(iter, 3)

        if type == "ttyS":
            self.serialButton.set_sensitive(gtk.TRUE)
        else:
            self.serialButton.set_sensitive(gtk.FALSE)

        if emu3 == 0:
            self.Xemu3CheckButton.set_active(gtk.FALSE)
        else:
            self.Xemu3CheckButton.set_active(gtk.TRUE)
            
    def setupDeviceList(self):
	deviceList = ((_("/dev/ttyS0 (COM1 under DOS)"), "ttyS0" ),
                      (_("/dev/ttyS1 (COM2 under DOS)"), "ttyS1" ),
                      (_("/dev/ttyS2 (COM3 under DOS)"), "ttyS2" ),
                      (_("/dev/ttyS3 (COM4 under DOS)"), "ttyS3" ))
        
	for descrip, dev in deviceList:
            iter = self.deviceStore.append()
            self.deviceStore.set_value(iter, 0, descrip)
            self.deviceStore.set_value(iter, 1, dev)
        self.deviceStore.set_sort_column_id(0, gtk.SORT_ASCENDING)
        iter  = self.deviceStore.get_iter_first()
        self.deviceView.get_selection().select_iter(iter)

    def setCurrent(self, currentDev, currentMouse, emulate3, recenter=1):
        self.ignoreEvents = 1
        self.currentMouse = currentMouse

        parent = None
        iter = self.mouseStore.get_iter_first()
        # iterate over the list, looking for the current mouse selection
        while iter:
            # if this is a parent node, get the first child and iter over them
            if self.mouseStore.iter_has_child(iter):
                parent = iter
                iter = self.mouseStore.iter_children(parent)
                continue
            # if it's not a parent node and the mouse matches, select it.
            elif self.mouseStore.get_value(iter, 1) == currentMouse:
		if parent:
		    path = self.mouseStore.get_path(parent)
		    self.mouseView.expand_row(path, gtk.TRUE)
                selection = self.mouseView.get_selection()
                selection.unselect_all()
                selection.select_iter(iter)
                path = self.mouseStore.get_path(iter)
                col = self.mouseView.get_column(0)
                self.mouseView.set_cursor(path, col, gtk.FALSE)
                if recenter:
                    self.mouseView.scroll_to_cell(path, col, gtk.TRUE,
                                                  0.5, 0.5)
                break
            # get the next row.
            iter = self.mouseStore.iter_next(iter)
            # if there isn't a next row and we had a parent, go to the node
            # after the parent we've just gotten the children of.
            if not iter and parent:
                parent = self.mouseStore.iter_next(parent)
                iter = parent

        self.Xemu3CheckButton.set_active(emulate3)
        self.ignoreEvents = 0
    
    def launch(self, doDebug=None):
        self.doDebug = doDebug
        if doDebug:
            print "in mouse launch"
        self.setupScreen()
        return FirstbootGuiWindow.launch(self)

    def apply(self, *args):
        model, iter = self.mouseView.get_selection().get_selected()
        selectedMouse = model.get_value(iter,1)
        (gpm, xprotocol, device, emulate, shortname) = self.mouse.available()[selectedMouse]

        if emulate:
            Xemu3 = "yes"
        else:
            Xemu3 = "no"

        if self.Xemu3CheckButton.get_active() == gtk.TRUE:
            Xemu3 = "yes"

        self.writeMouseFile(selectedMouse, gpm, Xemu3, shortname)
        self.modifyXconfig(xprotocol, device, Xemu3)
        self.restartGpm()

        try:
            #If we're in reconfig mode, this will fail because there is no self.mainWindow
            self.mainWindow.destroy()
        except:
            pass            
        return 1

    def writeMouseFile(self, name, gpm, Xemu3, shortname):
        if self.doDebug:
            #If we're in debug mode, just print what we would otherwise write to the file
            print ('FULLNAME="%s"' %name)
            print ('MOUSETYPE="%s"' %gpm)
            print ('XEMU3="%s"' %Xemu3)
            print ('XMOUSETYPE="%s"' %shortname)        
        else:
            #This is the real thing, so write the file
            fd = open('/etc/sysconfig/mouse', 'w')
            fd.write('FULLNAME="%s"\n' %name)
            fd.write('MOUSETYPE="%s"\n' %gpm)
            fd.write('XEMU3="%s"\n' %Xemu3)
            fd.write('XMOUSETYPE="%s"\n' %shortname)
            fd.close()

    def modifyXconfig(self, xprotocol, device, Xemu3):
        if device == "ttyS":
            data, iter = self.deviceView.get_selection().get_selected()
            port = self.deviceStore.get_value(iter, 1)
            device = "/dev/%s" % port
        else:
            #We need the device to start with "/dev/"
            device = "/dev/%s" % device
            
        #import xf86config and make the necessary changes
        import xf86config
        (xconfig, xconfigpath) = xf86config.readConfigFile()
        Xmouse = xf86config.getCorePointer(xconfig)
        for o in Xmouse.options:
            if o.name == "Protocol":
                o.val = xprotocol
            if o.name == "Device":
                o.val = device
            if o.name == "Emulate3Buttons":
                o.val = Xemu3

            #Write out the modified Xconfig file
            xconfig.write(xconfigpath)

    def restartGpm(self):
        os.system("/sbin/service gpm restart")
        
    def stand_alone(self, doDebug = None):
        self.doDebug = doDebug
        self.setupScreen()
        return FirstbootGuiWindow.stand_alone(self, MouseWindow.windowTitle)

childWindow = MouseWindow
